
	program toy_missile

!---------------------------------------------------------------------------------------
!  Paul M. Alsing
!  Albuquerque High Performance Computing Center (AHPCC)
!  1601 Central NE
!  University of New Mexioc
!  Albuquerque, NM 87131
!  505-277-9094 / alsing@arc.unm.edu
!---------------------------------------------------------------------------------------
!
!  DATE: 	14 July 98	
!  PURPOSE:	f90 code for generating {x,y,z,data}  coordinate and data values,
!  		and connectivity information, for AVS 'Connectivity Example'
!		AHPCC AVS Workshop 14 July 98
!
!---------------------------------------------------------------------------------------
!  Discussion:
!  -----------
!  We want to contruct a cylindrically symmetric missle and
!  put some data on its surface. We need to specify the connectivity
!  of the points on the missile, and this is best done from cylindrical
!  coordinates.  However, the actual coordinates we give to AVS
!  is best done in cartesian coordinates {x,y,z}, so we must convert
!  from cylindrical coordinates {r,theta,z}.
!
!  The missile has lenth L=10. along the z-axis with z=0 being the tail
!  end of the missle and z=L being the tip of the missile.
!  For 0<z<L/4 we vary the cylindrical radius of the missile from
!  rho_max=1.3 down to a radius of 1.  This makes the flared tail of the
!  missile. For L/4<z<8 the body of the missile is a cylinder of radius 1.  
!  For 8<z<L. we vary the radius of the missile from 1 to 0, to make the
!  cone shaped missile head.
!
!  To specify the connectivity we need to take 2 circles at z and z+dz and
!  list triplets of points that 'thread' back an forth between the two circles,
!  like the webbing on the side of an indian drum.  Lets pick two adjacent circles
!  at z and dz and flatten the area between them into a long rectangular strip;
!
! 
!									 z
! n+N   n+N+1   n+N+2     n+N+3   ...          ...               n+2N-1  ^
!  |-------x-------x-------|-------|-------|-------|-------|-------|	 |
!          |	   |							 |
!	   |	   |							 |
!  |-------x-------x-------|-------|-------|-------|-------|-------|	  ---> theta
!  n      n+1     n+2     n+3   ...              ...              n+N-1
!
!
!  The label 'n',..., represents the nth entry (row) in the {x,y,z,data} file, 
!  ie. the point 'n'.  Lets look at the 4 points indicated by the 'x's above.  
!  Since we are going to triangulate the surface of the missile we need to thread
!  these 4 points in a right-hand rule fashion which will yield 2 oriented
!  triangles.  Listing their vertices, we have {n+1,n+2,n+N+1} as the first
!  oriented triangle and {n+2,n+N+2,n+N+1} as the second oriented triangle
!  in the quadralateral formed by the 4 'x's above.  In AVS we only need to
!  specify these as one single long array, so the connectivity will be specified as
!  the single array {.......,n+1,n+2,n+N+1,n+2,n+N+2,n+N+1,.......}.
!  For a given pair of circles at z and z+dz, we traverse the above 'rectangular strip'
!  in the increasing theta direction, and specify the 2 triplets for the pair of 
!  oriented triangles in each group of 4 points which form a quadrateral. Once this
!  pair of circles is complete, we grab the next pair of cirles at z+dz and z+2dz,
!  as we proceed up the axis of the cylinder, until all pairs are exhausted.  
!---------------------------------------------------------------------------------------
	
	implicit none
	

!  Variables
!  N 	# pts on a circle
!  M	# circles
!  L	# length of cylinder

	integer, parameter		:: N=50,M=50
	real,    parameter		:: L=10.
        real 				:: r=1.	
        real				:: rho_max = 1.3
  	real				:: z_tail  !where tail fuses to body of cyl
         
	integer				:: i,j,k
	real				:: x,y,z,theta,pi,rho
	
	
	pi = 4.*atan(1.)
	z_tail = L/4.			!tail of missle is from z=0->z_tail
		
! open output files

	open(1,file='toy_missile.coordsanddata')
	open(2,file='toy_missile.connectivity')

	
! move up the z axis of the cylinder and create x,y,z values
! Variables
!  N 		# pts on a circle
!  M		# circles (each cirle at a constant value of z)
!  L		# length of missile
!  rho    	fixed radius of the cylindrical body of the missile
!  rho_max	maximum radius of the flared tail of the missile
!  z_tail	position at which tail of missile joins its cylindrical body	

!---------------------------------------------------------------------------------------
!  			Coordinates and Node Data Information
!---------------------------------------------------------------------------------------

        rho=r			!rho is a fixed radius, r is a varying radius

        write(1,*)N*(M+1)	!write the # of rows of {x,y,z,data} to be computed
	do k = 0, M
	   z = L/M*k		!z(k=0)=0, z(k=M)=L
	   if(L-z.lt.2*rho) then
	      r=(L-z)/2.	!shrink radius for the cone shaped nose of the missle
	   else
	      r=rho		!cylindrical body of missile has constant radius		
           endif
           			!tail of missile has radius rho<r<rho_max
          if(z < z_tail) r = rho_max + (rho-rho_max)*(z/z_tail)
          	
	   do j = 0, N-1			
	      theta = 2.*pi*j/N
	       x = r*cos(theta)
	       y = r*sin(theta)
	       !write out {x,y,z,data} where data is some interesting function
	       write(1,*) x, y, z, 127.*(1.+sin(2*theta)*sin(2*pi*z/L*2))
	   enddo
	enddo 
	
!---------------------------------------------------------------------------------------
!  				Connectivity Information
!---------------------------------------------------------------------------------------

!  move up the z axis and create the connectivity, using right
!  hand rule.  For the above double do loop we have the single
!  index A=k*N+j.  Points are connected as 2 triangles per square
!  (1) {P(k,j), P(k+1,j+1), P(k+1,j)} and
!  (2) {P(k,j), P(k,j+1), P(k+1,j+1)}
!  (see the Discussion at the top of the code)
!  Note: we use the f90 intrinsic modulo to take care of the periodicity in
!        the theta (or j index) direction.

        write(2,*)6*N*M
	do k = 0, M-1
	   do j = 0, N-1
	      write(2,*) k*N+j
	      write(2,*) k*N+modulo(j+1,N)
	      write(2,*) (k+1)*N + j
	      write(2,*) k*N+modulo(j+1,N)
	      write(2,*) (k+1)*N + modulo(j+1,N)
	      write(2,*) (k+1)*N + j
	   enddo
	enddo
	
	end program toy_missile
	
	
	
	
	
